# Code used for analysis:

# Jansen, Lloyd, Oschadleus and Altwegg. 2019. Survival synchronicity in two avian insectivore communities. Ibis.


##################################################################################
# Hierarchical capture-recapture/resight model with a species-specific intercept
# Separate parts for Darvill and Koeberg data, the model is not site-specific
# Additive random effect per species to study the asynchronous temporal component of the
# variation in survival 
# Additive random effect common to all species to study the synchronous temporal component of
# the variation in survival 
# The two survival random effects are assumed to be uncorrelated
# Random temporal variation per species in the recapture probability
##################################################################################

library(R2WinBUGS)

################################START Darvill script##################################

##################################################################################
# Create vector of species for the Darvill analysis - used as species index and in the estimation of the # intra-class correlation coefficient (ICC)
##################################################################################

# sorted by ring, excl. initial capture in the last capture occasion

spp<-read.csv("darvill - 1999-2013 Oct 4 species, SSM use.csv") 

library(pastecs)
ring.spp=aggregate(spp$spp,by=list(spp$ring), FUN=first, na.rm=TRUE)
ring.spp[,3] = 0
for (i in 1:length(ring.spp[,2])){
  (if (isTRUE(ring.spp[i,2] == 604))
    ring.spp[i,3] = 1)
}
for (i in 1:length(ring.spp[,2])){
  (if (isTRUE(ring.spp[i,2] == 646))
    ring.spp[i,3] = 2)
}
for (i in 1:length(ring.spp[,2])){
  (if (isTRUE(ring.spp[i,2] == 649))
    ring.spp[i,3] = 3)
}
for (i in 1:length(ring.spp[,2])){
  (if (isTRUE(ring.spp[i,2] == 791))
    ring.spp[i,3] = 4)
}

species=ring.spp[,3]
g=length(unique(species))


# Import capture data, 1 = captured in t, 0 = not captured in t
CH <- as.matrix(read.table(file = "darvill - 1999-2013 Oct 4 species, SSM use.inp", sep = " "))

# Create vector with occasion of marking
get.first <- function(x) min(which(x!=0))
f <- apply(CH, 1, get.first)

################################END Darvill script####################################

################################START Koeberg script#################################

##################################################################################
# Create vector of species for the Koeberg analysis - used as species index and in the estimation of 
# the # intra-class correlation coefficient (ICC)
##################################################################################

spp1<-read.csv("koeberg 2000-2007 - use.csv") 	# incl initial capture in the last occasion
spp=spp1[order(spp1$ring),]

spp$num.sp = 0
for (i in 1:length(spp$ring)){
  (if (isTRUE(spp$sp[i] == "BTAP"))
    spp$num.sp[i] = 1)
} #i
for (i in 1:length(spp$ring)){
  (if (isTRUE(spp$sp[i] == "KAPR"))
    spp$num.sp[i] = 2)
} #i
for (i in 1:length(spp$ring)){
  (if (isTRUE(spp$sp[i] == "KARO"))
    spp$num.sp[i] = 3)
} #i	

species=spp$num.sp
g=length(unique(species))

# Import data, 1 = captured/resighted in t, 0 = not captured/resighted in t
CH <- as.matrix(read.table(file = "koeberg 2000-2007 - 3 species, SSM use.inp", sep = " "))

# Create vector with occasion of marking
get.first <- function(x) min(which(x!=0))
f <- apply(CH, 1, get.first)

################################END Koeberg script#################################


############################
# Specify model in BUGS language
############################

sink("syncmodel.bug")
cat("
model {
######################
# Parameters:
# phi: survival probability
# p: recapture probability
###################### 

####################
# Priors and constraints
####################

for (i in 1:nind){  
    for (t in f[i]:(n.occasions-1)){			
    	logit(phi[i,t]) <- lphi[group[i],t]  		# survival
 	logit(p[i,t]) <- lp[group[i],t] 		# recapture
	} #t
} #i

for (u in 1:g){		
	for (t in 1:(n.occasions-1)){
 	lphi[u,t] <- lmu.phig[u] + beta.phi[u,t] + eps.phi[t] 
        	beta.phi[u,t] ~ dnorm(0, tau.phig[u])I(-5,5)  				
	lp[u,t] <- lmu.p + eps.p[u,t] 
	eps.p[u,t] ~ dnorm(0, tau.p)I(-5,5)
	} #t	   	
} #u
     	
for (t in 1:(n.occasions-1)){	
	eps.phi[t] ~ dnorm(0, tau.phit)I(-5,5)  
} #t

for (u in 1:g){
	mu.phig[u] ~ dunif(0,1)			# mean survival per species
	lmu.phig[u] <- log(mu.phig[u]/(1-mu.phig[u]))
	sig.phig[u] ~ dunif(0,5)                               	# standard deviation survival per species
	tau.phig[u] <- pow(sig.phig[u], -2)                    # = 1/sig.phig[u] squared
      sig2.phig[u] <- pow(sig.phig[u], 2)                    # variance survival per species
} #u
  
sig.phit ~ dunif(0,5)				# common temporal standard deviation survival
tau.phit <- pow(sig.phit, -2)
sig2.phit <- pow(sig.phit, 2)			# common temporal variance survival

   
mu.p ~ dunif(0,1)				# mean recapture
lmu.p <- log(mu.p/(1-mu.p))
sig.p ~ dunif(0,5)				# standard deviation recapture
tau.p <- pow(sig.p, -2) 
sig2.p <- pow(sig.p, 2) 				# variance recapture

# derived parameters (intra-class correlation coefficient)
for (u in 1:g){	
	for (t in 1:(n.occasions-1)){	 
	gt.phi[u,t] <- exp(lmu.phig[u] + beta.phi[u,t] + eps.phi[t])/(1 + exp(lmu.phig[u] + beta.phi[u,t] + 
	eps.phi[t])) 
	} #t
	icc[u] <- sig2.phit/(sig2.phit + sig2.phig[u])
} #u

# State-space model likelihood
for (i in 1:nind){
	# Define latent state at first capture
	z[i,f[i]] <- 1
	for (t in (f[i]+1):n.occasions){
		# State process
		z[i,t] ~ dbern(mu1[i,t])
		mu1[i,t] <- phi[i,t-1] * z[i,t-1]
		# Observation process
		Y[i,t] ~ dbern(mu2[i,t])
		mu2[i,t] <- p[i,t-1] * z[i,t]
	} #t
} #i
}
",fill = TRUE)
sink()

# Function to create a matrix with information about known latent state z
known.state.cjs <- function(ch){
  state <- ch
  for (i in 1:dim(ch)[1]){
    n1 <- min(which(ch[i,]==1))
    n2 <- max(which(ch[i,]==1))
    state[i,n1:n2] <- 1
    state[i,n1] <- NA      			# because CJS conditional on first capture
  } #i
  state[state==0] <- NA
  return(state)
}

# Bundle data
bugs.data <- list(Y = CH, f = f, nind = dim(CH)[1], n.occasions = dim(CH)[2], z = known.state.cjs(CH), g=g, group=species)

# Function to create a matrix of initial values for latent state z
cjs.init.z <- function(ch,f){
  for (i in 1:dim(CH)[1]){
    if (sum(ch[i,])==1) next
    n2 <- max(which(ch[i,]==1))
    ch[i,f[i]:n2] <- NA
  } #i
  for (i in 1:dim(CH)[1]){
    ch[i,1:f[i]] <- NA
  } #i
  return(ch)
}

# Initial values
inits <- function(){list(z = cjs.init.z(CH, f), mu.phig = runif(g, 0, 1), 	mu.p = runif(1, 0, 1), 
                         sig.phig = runif(g, 0, 2), sig.phit = runif(1, 0, 2), sig.p = runif(1, 0, 2))}

# Parameters monitored
parameters <- c("mu.phig", "gt.phi", "sig2.phig", "sig2.phit", "icc", "mu.p", "sig2.p")

# MCMC settings
ni <- 100000
nt <- 20						# For Koeberg analysis nt=10
nb <- 50000
nc <- 3

# Call WinBUGS from R
syncmodel <- bugs(bugs.data, inits, parameters, "syncmodel.bug", n.chains = nc, n.thin = nt, 
                  n.iter = ni, n.burnin = nb, debug = FALSE, bugs.directory =bugs.dir, working.directory = getwd())

# Summarize posteriors
save.image()
save(syncmodel, file="syncmodel.Rdata")
print(syncmodel, digits = 3)

# Checking for convergence
outmc <- as.mcmc.list(syncmodel)
plot(outmc, ask=FALSE)           
autocorr.plot(outmc, ask=FALSE)

#######################################END#######################################
